package org.greenstone.gatherer.util;
import java.awt.event.*;
import javax.swing.*;
/** Just like any other button group, this object encapsulates several checkbox listeners, but instead of recording which are selected, or indeed what buttons it has assigned to it, this group has but one purpose: to ensure that at least one check button is always selected. */
public class XORToggleButtonGroup
    implements ActionListener {
    /** true if we are to ignore any further change events we recieve. */
    private boolean ignore = false;
    /** The current number of checkboxes that are selected. Never < 1. */
    private int selected_count = 0;

    private JToggleButton last_button;

    /** Any implementation of ActionListener includes this method so that we can be informed when an action has occured on any registered checkboxes, allowing us to roll back the action if this is the only selected checkbox remaining.
	  * @param event An <strong>ActionEvent</strong> containing information about the event.
	  */
    public void actionPerformed(ActionEvent event) {
	if(!ignore) {
	    ignore = true;
	    JToggleButton toggle_button = (JToggleButton) event.getSource();
	    if(toggle_button.isSelected()) { 
		selected_count++;
	    }
	    else {
		if(selected_count > 1) {
		    selected_count--;
		}
		else {
		    toggle_button.setSelected(true);
		}
	    }
	    ignore = false;
	}
    }
    public void add(JToggleButton toggle_button) {
	last_button = toggle_button;
	// If the checkbox is selected, increase our count.
	if(toggle_button.isSelected()) {
	    selected_count++;
	}
	toggle_button.addActionListener(this);
    }

    /** Ensures that at least one option is selected. */
    public void establish() {
	if(selected_count == 0 && last_button != null) {
	    last_button.setSelected(true);
	}
    }
}
