package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;
import java.util.Enumeration;

/** core class for web interface generation. Receives requests consisting
 * of an xml representation of cgi args, and returns the page of data - in 
 * html by default. The requests are processed by the appropriate action class
 *
 * @see Action
 */
public class WebReceptionist extends TransformingReceptionist {

    /** the set up variables */

    /** the cgi args converter */
    protected GSCGI cgi = null;

    public void setCGI(GSCGI cgi) {
	this.cgi=cgi;
    }
    
    protected void preProcessRequest(Element request) {
	// convert cgi short names to long names in the param list
	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list!=null) {
	    if (!this.cgi.paramListToLongNames(param_list)) {
		System.err.println("Receptionist Error: couldn't convert short names to long names in the param list!");
		
	    }
	}
    }

    // need to modify some param lists
    protected Element transformPage(Element page) {
	
	Element page_response = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_RESPONSE_ELEM);
	// look for a service description
	Element service = (Element)GSXML.getChildByTagName(page_response, GSXML.SERVICE_ELEM);
	if (service != null) {
	    // look for a param-list
	    Element param_list = (Element)GSXML.getChildByTagName(service, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	    if (param_list != null) {

		// add in cgi short names
		this.cgi.paramListAddShortNames(param_list);
	    }
	}
	return super.transformPage(page);
    }

}
