###########################################################################
#
# win32 makefile -- mgpp\lib
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the
# University of Waikato, New Zealand.
#
# Copyright (C) 1999  The New Zealand Digital Library Project
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

MGPPHOME = ..

AR = lib
CC = cl
CPPFLAGS = -GX
LINK = cl

DEFS = -DQUIET -DHAVE_CONFIG_H -D__WIN32__ -DSHORT_SUFFIX -D__STDC__ -D__GNU_LIBRARY__ -D__MSDOS__
INCLUDES = -I"." -I"$(MGPPHOME)" -I"$(MGPPHOME)\lib"
LIBS = $(MGPPHOME)\lib\libmgpp.lib

COMPILE = $(CC) -c $(DEFS) $(INCLUDES) $(CPPFLAGS)

.SUFFIXES:
.SUFFIXES: .c .cpp .obj
.c.obj:
	$(COMPILE) $<
.cpp.obj:
	$(COMPILE) $<

ANSI2KNR = 
o = .obj

HEADERS = \
  FIvfLevelInfo.h     TextEl.h            hash.h              stemmer.h  \
  FText.h             TextGet.h           invf.h              text.h     \
  FileBuf.h           UCArray.h           locallib.h          words.h    \
  FragLevelConvert.h  WordData.h          mg.h                           \
  IndexData.h         build.h             mg_errors.h                    \
  TagInfo.h           comp_dict.h         mg_files.h          Weights.h  \
  MGQuery.h           Terms.h             QueryLex.h          QueryParser.h \
  GSDLQueryLex.h	GSDLQueryParser.h


SOURCES = \
  FIvfLevelInfo.cpp        ivf.pass1.cpp            mgpp_invf_dump.cpp       \
  FText.cpp                ivf.pass2.cpp            mgpp_passes.cpp          \
  FileBuf.cpp              locallib.cpp             mgpp_perf_hash_build.cpp \
  FragLevelConvert.cpp     mg.special.cpp           mgpp_stem_idx.cpp        \
  IndexData.cpp            mgpp_compression_dict.cpp  mgpp_weights_build.cpp   \
  TagInfo.cpp              mgpp_decompress_text.cpp   stemmer.cpp            \
  TextEl.cpp               mg_errors.cpp            text.cpp               \
  TextGet.cpp              mgpp_fast_comp_dict.cpp    text.pass1.cpp         \
  UCArray.cpp              mg_files.cpp             text.pass2.cpp         \
  comp_dict.cpp            mgpp_invf_dict.cpp         words.cpp              \
  invf.cpp                 mgpp_invf_dict_dump.cpp    Weights.cpp            \
  MGQuery.cpp              Terms.cpp                QueryTester.cpp        \
  QueryLex.cpp             QueryParser.cpp         \
  GSDLQueryLex.cpp	   GSDLQueryParser.cpp      Queryer.cpp


EXEC          = mgpp_passes mgpp_decompress_text mgpp_perf_hash_build \
                mgpp_invf_dump mgpp_weights_build mgpp_invf_dict mgpp_stem_idx \
                mgpp_invf_dict_dump mgpp_compression_dict mgpp_fast_comp_dict \
                QueryTester Queryer

DISTFILES = Makefile.in $(HEADERS) $(SOURCES)
 
all : $(EXEC) libtextin.lib


PASSES_OBJS = \
	mgpp_passes$o text.pass1$o comp_dict$o \
	text.pass2$o locallib$o \
	ivf.pass1$o ivf.pass2$o mg.special$o mg_files$o \
	words$o invf$o FileBuf$o TextEl$o UCArray$o TagInfo$o \
        FText$o text$o FIvfLevelInfo$o

mgpp_passes: $(PASSES_OBJS)
	$(LINK) $(PASSES_OBJS) $(LIBS)

QUERYER_OBJS = Queryer$o mg_files$o mg_errors$o locallib$o invf$o UCArray$o \
               FIvfLevelInfo$o FragLevelConvert$o Terms$o MGQuery$o \
               IndexData$o stemmer$o Weights$o TextGet$o text$o FText$o \
               GSDLQueryParser$o GSDLQueryLex$o words$o

Queryer: $(QUERYER_OBJS)
	$(LINK) $(QUERYER_OBJS) $(LIBS)

QUERY_TESTER_OBJS = QueryTester$o mg_files$o invf$o UCArray$o \
                    FIvfLevelInfo$o FragLevelConvert$o Terms$o MGQuery$o \
                    IndexData$o stemmer$o Weights$o

QueryTester: $(QUERY_TESTER_OBJS)
	$(LINK) $(QUERY_TESTER_OBJS) $(LIBS)

WEIGHTS_BUILD_OBJS = mgpp_weights_build$o mg_files$o invf$o UCArray$o \
                     FIvfLevelInfo$o FragLevelConvert$o

mgpp_weights_build: $(WEIGHTS_BUILD_OBJS)
	$(LINK) $(WEIGHTS_BUILD_OBJS) $(LIBS)

COMP_DICT_OBJS = mgpp_compression_dict$o mg_files$o locallib$o

mgpp_compression_dict: $(COMP_DICT_OBJS)
	$(LINK) $(COMP_DICT_OBJS) $(LIBS)

FAST_COMP_OBJS = mgpp_fast_comp_dict$o mg_files$o locallib$o

mgpp_fast_comp_dict: $(FAST_COMP_OBJS)
	$(LINK) $(FAST_COMP_OBJS) $(LIBS)

INVF_DUMP_OBJS = mgpp_invf_dump$o locallib$o mg_files$o invf$o UCArray$o

mgpp_invf_dump: $(INVF_DUMP_OBJS)
	$(LINK) $(INVF_DUMP_OBJS) $(LIBS)

INVF_DICT_OBJS = mgpp_invf_dict$o mg_files$o invf$o UCArray$o

mgpp_invf_dict: $(INVF_DICT_OBJS)
	$(LINK) $(INVF_DICT_OBJS) $(LIBS)

INVF_DICT_DUMP_OBJS = mgpp_invf_dict_dump$o mg_files$o invf$o UCArray$o

mgpp_invf_dict_dump: $(INVF_DICT_DUMP_OBJS)
	$(LINK) $(INVF_DICT_DUMP_OBJS) $(LIBS)

PERF_HASH_OBJS = mgpp_perf_hash_build$o mg_files$o invf$o UCArray$o

mgpp_perf_hash_build: $(PERF_HASH_OBJS)
	$(LINK) $(PERF_HASH_OBJS) $(LIBS)

STEM_IDX_OBJS = mgpp_stem_idx$o mg_files$o stemmer$o \
                invf$o UCArray$o

mgpp_stem_idx: $(STEM_IDX_OBJS)
	$(LINK) $(STEM_IDX_OBJS) $(LIBS)

DECOMPRESS_TEXT_OBJS = \
                mgpp_decompress_text$o mg_files$o mg_errors$o \
                locallib$o text$o UCArray$o TextGet$o FText$o

mgpp_decompress_text: $(DECOMPRESS_TEXT_OBJS)
	$(LINK) $(DECOMPRESS_TEXT_OBJS) $(LIBS)


#mg_errors$o removed from LIB_OBJS to avoid conflict with mg's libtextin.a
LIB_OBJS = mg_files$o  mg_errors$o locallib$o invf$o UCArray$o \
           FIvfLevelInfo$o FragLevelConvert$o Terms$o MGQuery$o \
           IndexData$o stemmer$o Weights$o TextGet$o text$o FText$o \
           GSDLQueryParser$o GSDLQueryLex$o words$o

libtextin.lib: $(LIB_OBJS)
        if exist libtextin.lib del libtextin.lib
        $(AR) /out:libtextin.lib $(LIB_OBJS)

install: 
clean:
	del *$o 
        del libtextin.lib
